@@ -3,7 +3,7 @@ NULL
 
 #' Get/set years component of a date-time.
 #'
-#' Date-time must be a POSIXct, POSIXlt, Date, chron, yearmon, yearqtr, zoo, 
+#' Date-time must be a POSIXct, POSIXlt, Date, Period, chron, yearmon, yearqtr, zoo, 
 #' zooreg, timeDate, xts, its, ti, jul, timeSeries, and fts objects. 
 #'
 #' year does not yet support years before 0 C.E.
@@ -11,6 +11,7 @@ NULL
 #' @export year "year<-"
 #' @aliases year year<-
 #' @S3method year default
+#' @S3method year Period
 #' @param x a date-time object   
 #' @return the years element of x as a decimal number
 #' @keywords utilities manip chron methods
@@ -25,5 +26,17 @@ year <- function(x)
 year.default <- function(x)
     as.POSIXlt(x, tz = tz(x))$year + 1900
 
+year.Period <- function(x)
+  slot(x, "year")
+
 "year<-" <- function(x, value)
-  x <- x + years(value - year(x))
\ No newline at end of file
+  x <- x + years(value - year(x))
+
+
+setGeneric("year<-")
+
+#' @export
+setMethod("year<-", signature("Period"), function(x, value){
+  slot(x, "year") <- value
+  x
+})
