@@ -1,149 +1,143 @@
 context("Parsing")
 
 test_that("ymd functions correctly parse dates separated by -", {
-  expect_that(ymd("2010-01-02"), 
+  expect_that(ymd("2010-01-02"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(ymd("10-01-02"), 
+  expect_that(ymd("10-01-02"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(ydm("2010-02-01"), 
+  expect_that(ydm("2010-02-01"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(mdy("01-02-2010"), 
+  expect_that(mdy("01-02-2010"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(myd("01-2010-02"), 
+  expect_that(myd("01-2010-02"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(dmy("02-01-2010"), 
+  expect_that(dmy("02-01-2010"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(dym("02-2010-01"), 
+  expect_that(dym("02-2010-01"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(ymd(c("2010-01-02","2010-01-03")) , 
-    equals(as.POSIXct(c("2010-01-02","2010-01-03"), tz = 
+  expect_that(ymd(c("2010-01-02","2010-01-03")) ,
+    equals(as.POSIXct(c("2010-01-02","2010-01-03"), tz =
     "UTC")))
 })
 
 test_that("ymd functions correctly parse dates separated by /", {
-  
-  expect_that(ymd("2010/01/02"), 
+  expect_that(ymd("2010/01/02"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(ymd("10/01/02"), 
+  expect_that(ymd("10/01/02"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(ydm("2010/02/01"), 
+  expect_that(ydm("2010/02/01"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(mdy("01/02/2010"), 
+  expect_that(mdy("01/02/2010"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(myd("01/2010/02"), 
+  expect_that(myd("01/2010/02"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(dmy("02/01/2010"), 
+  expect_that(dmy("02/01/2010"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(dym("02/2010/01"), 
+  expect_that(dym("02/2010/01"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(ymd(c("2010/01/02","2010/01/03")) , 
-    equals(as.POSIXct(c("2010-01-02","2010-01-03"), tz = 
+  expect_that(ymd(c("2010/01/02","2010/01/03")) ,
+    equals(as.POSIXct(c("2010-01-02","2010-01-03"), tz =
     "UTC")))
 })
 
 
 test_that("ymd functions correctly parse dates separated by .", {
-  
-  expect_that(ymd("2010.01.02"), 
+  expect_that(ymd("2010.01.02"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(ymd("10.01.02"), 
+  expect_that(ymd("10.01.02"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(ydm("2010.02.01"), 
+  expect_that(ydm("2010.02.01"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(mdy("01.02.2010"), 
+  expect_that(mdy("01.02.2010"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(myd("01.2010.02"), 
+  expect_that(myd("01.2010.02"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(dmy("02.01.2010"), 
+  expect_that(dmy("02.01.2010"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(dym("02.2010.01"), 
+  expect_that(dym("02.2010.01"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(ymd(c("2010.01.02","2010.01.03")) , 
-    equals(as.POSIXct(c("2010-01-02","2010-01-03"), tz = 
+  expect_that(ymd(c("2010.01.02","2010.01.03")) ,
+    equals(as.POSIXct(c("2010-01-02","2010-01-03"), tz =
     "UTC")))
 })
 
 test_that("ymd functions correctly parse dates separated by ,", {
-  
-  expect_that(ymd("2010,01,02"), 
+  expect_that(ymd("2010,01,02"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(ymd("10,01,02"), 
+  expect_that(ymd("10,01,02"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(ydm("2010,02,01"), 
+  expect_that(ydm("2010,02,01"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(mdy("01,02,2010"), 
+  expect_that(mdy("01,02,2010"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(myd("01,2010,02"), 
+  expect_that(myd("01,2010,02"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(dmy("02,01,2010"), 
+  expect_that(dmy("02,01,2010"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(dym("02,2010,01"), 
+  expect_that(dym("02,2010,01"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(ymd(c("2010,01,02","2010,01,03")) , 
-    equals(as.POSIXct(c("2010-01-02","2010-01-03"), tz = 
+  expect_that(ymd(c("2010,01,02","2010,01,03")) ,
+    equals(as.POSIXct(c("2010-01-02","2010-01-03"), tz =
     "UTC")))
 })
 
 test_that("ymd functions correctly parse dates separated by :", {
-  
-  expect_that(ymd("2010:01:02"), 
+  expect_that(ymd("2010:01:02"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(ymd("10:01:02"), 
+  expect_that(ymd("10:01:02"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(ydm("2010:02:01"), 
+  expect_that(ydm("2010:02:01"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(mdy("01:02:2010"), 
+  expect_that(mdy("01:02:2010"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(myd("01:2010:02"), 
+  expect_that(myd("01:2010:02"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(dmy("02:01:2010"), 
+  expect_that(dmy("02:01:2010"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(dym("02:2010:01"), 
+  expect_that(dym("02:2010:01"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(ymd(c("2010:01:02","2010:01:03")) , 
-    equals(as.POSIXct(c("2010-01-02","2010-01-03"), tz = 
+  expect_that(ymd(c("2010:01:02","2010:01:03")) ,
+    equals(as.POSIXct(c("2010-01-02","2010-01-03"), tz =
     "UTC")))
 })
 
 test_that("ymd functions correctly parse dates with no separators", {
-  
-  expect_that(ymd("20100102"), 
+  expect_that(ymd("20100102"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(ydm("20100201"), 
+  expect_that(ydm("20100201"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(mdy("01022010"), 
+  expect_that(mdy("01022010"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(myd("01201002"), 
+  expect_that(myd("01201002"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(dmy("02012010"), 
+  expect_that(dmy("02012010"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(dym("02201001"), 
+  expect_that(dym("02201001"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(ymd(c("20100102","20100103")) , 
-    equals(as.POSIXct(c("2010-01-02","2010-01-03"), tz = 
+  expect_that(ymd(c("20100102","20100103")) ,
+    equals(as.POSIXct(c("2010-01-02","2010-01-03"), tz =
     "UTC")))
-  expect_that(ymd("100102"), 
+  expect_that(ymd("100102"),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
 })
 
 test_that("ymd functions correctly parse dates with no separators and no quotes", {
-  
-  expect_that(ymd(20100102), 
+  expect_that(ymd(20100102),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(ydm(20100201), 
+  expect_that(ydm(20100201),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(mdy(01022010), 
+  expect_that(mdy(01022010),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(myd(01201002), 
+  expect_that(myd(01201002),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(dmy(02012010), 
+  expect_that(dmy(02012010),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(dym(02201001), 
+  expect_that(dym(02201001),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
-  expect_that(ymd(c(20100102, 20100103)) , 
-    equals(as.POSIXct(c("2010-01-02","2010-01-03"), tz = 
+  expect_that(ymd(c(20100102, 20100103)) ,
+    equals(as.POSIXct(c("2010-01-02","2010-01-03"), tz =
     "UTC")))
-  expect_that(ymd(100102), 
+  expect_that(ymd(100102),
     equals(as.POSIXct("2010-01-02", tz = "UTC")))
   expect_that(ymd_hms(20100102235959),
               equals(as.POSIXct("2010-01-02 23:59:59", tz = "UTC")))
@@ -191,13 +185,12 @@ test_that("ymd_hms correctly handles a variety of formats", {
     "2010-01-02 23:59:59", tz = "UTC")))
   expect_that(ymd_hms("2010:01:02-23:59:59"), equals(as.POSIXct(
     "2010-01-02 23:59:59", tz = "UTC")))
-
-  ## VC: Next doesn't parse on my machine. It's OS specific.
+  ## VS: Next doesn't parse on my machine. It's OS specific.
   ## expect_that(ymd_hms("2010-01-02 23:59:61"), equals(as.POSIXct(
-  ##   "2010-01-03 00:00:01", tz = "UTC")))  
-  ## expect_that(ymd_hms(c("2010-01-02 23:59:61", 
+  ##   "2010-01-03 00:00:01", tz = "UTC")))
+  ## expect_that(ymd_hms(c("2010-01-02 23:59:61",
   ##   "2010-01-02 00:00:00")), equals(as.POSIXct(
-  ##   c("2010-01-03 00:00:01", "2010-01-02 00:00:00"), tz = 
+  ##   c("2010-01-03 00:00:01", "2010-01-02 00:00:00"), tz =
   ##   "UTC")))
   expect_that(ymd_hms("10-01-02 23:59:59"), equals(as.POSIXct(
     "2010-01-02 23:59:59", tz = "UTC")))
@@ -205,33 +198,33 @@ test_that("ymd_hms correctly handles a variety of formats", {
 
 
 test_that("hms functions correctly handle : separators", {
-  expect_that(hms("3:3:3"), equals(hours(3) + minutes(3) + 
+  expect_that(hms("3:3:3"), equals(hours(3) + minutes(3) +
     seconds(3)))
-  expect_that(hms("03:03:03"), equals(hours(3) + minutes(3) + 
+  expect_that(hms("03:03:03"), equals(hours(3) + minutes(3) +
     seconds(3)))
-  expect_that(ms("03:03"), equals(minutes(3) + 
+  expect_that(ms("03:03"), equals(minutes(3) +
     seconds(3)))
   expect_that(hm("03:03"), equals(hours(3) + minutes(3)))
   expect_that(hm("03:3"), equals(hours(3) + minutes(3)))
 })
 
 test_that("hms functions correctly handle . separators", {
-  expect_that(hms("3.3.3"), equals(hours(3) + minutes(3) + 
+  expect_that(hms("3.3.3"), equals(hours(3) + minutes(3) +
     seconds(3)))
-  expect_that(hms("03.03.03"), equals(hours(3) + minutes(3) + 
+  expect_that(hms("03.03.03"), equals(hours(3) + minutes(3) +
     seconds(3)))
-  expect_that(ms("03.03"), equals(minutes(3) + 
+  expect_that(ms("03.03"), equals(minutes(3) +
     seconds(3)))
   expect_that(hm("03.03"), equals(hours(3) + minutes(3)))
   expect_that(hm("03.3"), equals(hours(3) + minutes(3)))
 })
 
 test_that("hms functions correctly handle - separators", {
-  expect_that(hms("3-3-3"), equals(hours(3) + minutes(3) + 
+  expect_that(hms("3-3-3"), equals(hours(3) + minutes(3) +
     seconds(3)))
-  expect_that(hms("03-03-03"), equals(hours(3) + minutes(3) + 
+  expect_that(hms("03-03-03"), equals(hours(3) + minutes(3) +
     seconds(3)))
-  expect_that(ms("03-03"), equals(minutes(3) + 
+  expect_that(ms("03-03"), equals(minutes(3) +
     seconds(3)))
   expect_that(hm("03-03"), equals(hours(3) + minutes(3)))
   expect_that(hm("03-3"), equals(hours(3) + minutes(3)))
@@ -239,11 +232,11 @@ test_that("hms functions correctly handle - separators", {
 
 
 test_that("hms functions correctly handle space separators", {
-  expect_that(hms("3 3 3"), equals(hours(3) + minutes(3) + 
+  expect_that(hms("3 3 3"), equals(hours(3) + minutes(3) +
     seconds(3)))
-  expect_that(hms("03 03 03"), equals(hours(3) + minutes(3) + 
+  expect_that(hms("03 03 03"), equals(hours(3) + minutes(3) +
     seconds(3)))
-  expect_that(ms("03 03"), equals(minutes(3) + 
+  expect_that(ms("03 03"), equals(minutes(3) +
     seconds(3)))
   expect_that(hm("03 03"), equals(hours(3) + minutes(3)))
   expect_that(hm("03 3"), equals(hours(3) + minutes(3)))
@@ -251,11 +244,11 @@ test_that("hms functions correctly handle space separators", {
 
 
 test_that("hms functions correctly handle , separators", {
-  expect_that(hms("3,3,3"), equals(hours(3) + minutes(3) + 
+  expect_that(hms("3,3,3"), equals(hours(3) + minutes(3) +
     seconds(3)))
-  expect_that(hms("03,03,03"), equals(hours(3) + minutes(3) + 
+  expect_that(hms("03,03,03"), equals(hours(3) + minutes(3) +
     seconds(3)))
-  expect_that(ms("03,03"), equals(minutes(3) + 
+  expect_that(ms("03,03"), equals(minutes(3) +
     seconds(3)))
   expect_that(hm("03,03"), equals(hours(3) + minutes(3)))
   expect_that(hm("03,3"), equals(hours(3) + minutes(3)))
@@ -263,18 +256,18 @@ test_that("hms functions correctly handle , separators", {
 
 
 test_that("hms functions correctly handle / separators", {
-  expect_that(hms("3/3/3"), equals(hours(3) + minutes(3) + 
+  expect_that(hms("3/3/3"), equals(hours(3) + minutes(3) +
     seconds(3)))
-  expect_that(hms("00/00/00"), equals(hours(0) + minutes(0) + 
+  expect_that(hms("00/00/00"), equals(hours(0) + minutes(0) +
     seconds(0)))
-  expect_that(ms("03/03"), equals(minutes(3) + 
+  expect_that(ms("03/03"), equals(minutes(3) +
     seconds(3)))
   expect_that(hm("03/03"), equals(hours(3) + minutes(3)))
   expect_that(hm("03/3"), equals(hours(3) + minutes(3)))
 })
 
 
-test_that("hms functions return NA on incompatible inputs", {
+test_that("hms functions return NA on shorter inputs", {
   expect_that(is.na(hms("3:3:3:4", quiet = TRUE)), is_true())
   expect_that(is.na(hms("03:03", quiet = TRUE)), is_true())
   expect_that(is.na(ms("03:02:01", quiet = TRUE)), is_true())
@@ -283,7 +276,7 @@ test_that("hms functions return NA on incompatible inputs", {
   expect_that(is.na(hm("03", quiet = TRUE)), is_true())
 })
 
-test_that("hms functions give warning on incompatible inputs", {
+test_that("hms functions give warning on shorter inputs", {
   expect_warning(hms("3:3:3:4"))
   expect_warning(hms("03:03"))
   expect_warning(ms("03:02:01"))
@@ -293,22 +286,21 @@ test_that("hms functions give warning on incompatible inputs", {
 })
 
 
-test_that("heterogeneous formats are correctly parsed", {  
-  expect_that(ymd(c(20090101, "2009-01-02", "2009 01 03", "2009-1-4",
-                    "2009-1, 5", "2009....1--6", "200901-07", "200901-8")),
-              equals(as.POSIXct(c("2009-01-01", "2009-01-02", "2009-01-03", "2009-01-04",
-                                  "2009-01-05", "2009-01-06", "2009-01-07", "2009-01-08"), tz = "UTC")))
+test_that("heterogeneous formats are correctly parsed", {
+  X <- c(20090101, "2009-01-02", "2009 01 03", "2009-1-4", "2009-1, 5", "2009....1--6", "200901-07", "200901-8")
+  Y <- c("2009-01-01", "2009-01-02", "2009-01-03", "2009-01-04", "2009-01-05", "2009-01-06", "2009-01-07", "2009-01-08")
+  expect_that( ymd(X) ,equals( as.POSIXct(Y, tz = "UTC")))
+  ## cbind(ymd(X), as.POSIXct(Y, tz = "UTC"))
 
-
-  expect_that({
-    x <- c(20100101120101, "2009-01-02 12-01-02", "2009.01.03 12:01:03", "2009-1-4 12-1-4",
-           "2009-1, 5 12:1, 5", "2009....1--6 - 12::1:6", "20090107 120107", "200901-08 1201-08",
-           "10-01-09 12:01:09", "10-01-10 10:01:10 AM", "10-01-11 10:01:11 PM")
-    ymd_hms(x)
-  }, equals(as.POSIXct(c("2010-01-01 12:01:01", "2009-01-02 12:01:02", "2009-01-03 12:01:03", 
-                         "2009-01-04 12:01:04", "2009-01-05 12:01:05", "2009-01-06 12:01:06", 
-                         "2009-01-07 12:01:07", "2009-01-08 12:01:08", "2010-01-09 12:01:09", 
-                         "2010-01-10 10:01:10", "2010-01-11 22:01:11"), tz = "UTC")))
+  X <- c(20100101120101, "2009-01-02 12-01-02", "2009.01.03 12:01:03", "2009-1-4 12-1-4",
+         "2009-1, 5 12:1, 5", "2009....1--6 - 12::1:6", "20090107 120107", "2009-01-08 1201-08",
+         "2010-01-09 12:01:09", "2010-01-10 10:01:10 AM", "2010-01-11 10:01:11 PM")
+  Y <- c("2010-01-01 12:01:01", "2009-01-02 12:01:02", "2009-01-03 12:01:03",
+         "2009-01-04 12:01:04", "2009-01-05 12:01:05", "2009-01-06 12:01:06",
+         "2009-01-07 12:01:07", "2009-01-08 12:01:08", "2010-01-09 12:01:09",
+         "2010-01-10 10:01:10", "2010-01-11 22:01:11")
+  expect_that( ymd_hms(X), equals(as.POSIXct(Y, tz = "UTC")))
+  cbind(as.character(ymd_hms(X)), as.character(as.POSIXct(Y, tz = "UTC")))
 })
 
 
@@ -316,7 +308,7 @@ test_that("truncated formats are correctly parsed", {
   expect_that({
     x <- c("2011-12-31 12:59:59", "2010-01-01 12:11", "2010-01-01 12", "2010-01-01")
     ymd_hms(x, truncated = 3)
-  }, equals(as.POSIXct(c("2011-12-31 12:59:59", "2010-01-01 12:11:00", "2010-01-01 12:00:00", 
+  }, equals(as.POSIXct(c("2011-12-31 12:59:59", "2010-01-01 12:11:00", "2010-01-01 12:00:00",
                          "2010-01-01 00:00:00"), tz = "UTC")))
 })
 
@@ -361,7 +353,7 @@ test_that( "Vectors of NA's are parsed as vectors of NA's", {
               , hour = c(0, 0, 0)
               , minute = c(NA_real_, NA_real_, NA_real_)
   )
-  
+
   expect_warning(ymd(NA, NA, NA))
   expect_equal(ymd(NA, NA, NA, quiet = TRUE), mna)
   expect_warning(ymd_hms(NA, NA, NA))
@@ -372,7 +364,7 @@ test_that( "Vectors of NA's are parsed as vectors of NA's", {
   expect_equal(hms(NA, NA, NA, quiet = TRUE), pna)
   expect_warning(ms(NA, NA, NA))
   expect_equal(ms(NA, NA, NA, quiet = TRUE), pna2)
-  
+
   expect_warning(ymd(c(NA, NA, NA)))
   expect_equal(ymd(c(NA, NA, NA), quiet = TRUE), mna)
   expect_warning(ymd_hms(c(NA, NA, NA)))
@@ -384,100 +376,165 @@ test_that( "Vectors of NA's are parsed as vectors of NA's", {
   expect_warning(ms(c(NA, NA, NA)))
   expect_equal(ms(c(NA, NA, NA), quiet = TRUE), pna2)
 })
-  
-  
-test_that("ISO8601: %z format (aka lubridate %Ot, %OO and %Oo formats) is correctly parsed",{
-  
+
+
+test_that("ISO8601: %z format (aka lubridate %Ou, %OO and %Oo formats) is correctly parsed",{
   expect_that(
     parse_date_time("2012-12-04 15:06:06.95-0800", "YmdHMOSz"),
-    equals(as.POSIXlt("2012-12-04 23:06:06.95 UTC", tz = "UTC")))
-  
+    equals(as.POSIXct("2012-12-04 23:06:06.95 UTC", tz = "UTC")))
   expect_that(
     parse_date_time(c("2012-12-04 15:06:06.95-08", "2012-12-04 15:06:06.95+08:00"), "YmdHMOSz"),
-    equals(as.POSIXlt(c("2012-12-04 23:06:06.95 UTC", "2012-12-04 07:06:06.95 UTC"), tz = "UTC")))
-  
+    equals(as.POSIXct(c("2012-12-04 23:06:06.95 UTC", "2012-12-04 07:06:06.95 UTC"), tz = "UTC")))
 })
 
 test_that("ISO8601: xxx_hms functions work correctly with z, Ou, OO and Oo formats.", {
-  
   expect_that(
-    ymd_hms(c("2013-01-24 19:39:07.880-0600", 
-              "2013-01-24 19:39:07.880", "2013-01-24 19:39:07.880-06:00", 
+    ymd_hms(c("2013-01-24 19:39:07.880-0600",
+              "2013-01-24 19:39:07.880", "2013-01-24 19:39:07.880-06:00",
               "2013-01-24 19:39:07.880-06", "2013-01-24 19:39:07.880Z")),
-    equals(as.POSIXct(c("2013-01-25 01:39:07.88 UTC", 
-                        "2013-01-24 19:39:07.88 UTC", "2013-01-25 01:39:07.88 UTC", 
+    equals(as.POSIXct(c("2013-01-25 01:39:07.88 UTC",
+                        "2013-01-24 19:39:07.88 UTC", "2013-01-25 01:39:07.88 UTC",
                         "2013-01-25 01:39:07.88 UTC", "2013-01-24 19:39:07.88 UTC"), tz = "UTC")))
 })
 
 
+test_that("ymd_hms treats time zones correctly", {
+  expect_that(ymd_hms("2012-03-03 23:06:07", tz="America/Chicago"),
+              equals(as.POSIXct("2012-03-03 23:06:07", tz = "America/Chicago")))
+})
+
 test_that("ymd_hms parses Ou format correctly ",{
   ## Correct usage
-  expect_that(ymd_hms("2012-03-04T05:06:07Z"), 
+  expect_that(ymd_hms("2012-03-04T05:06:07Z"),
               equals(ymd_hms("2012-03-04 05:06:07", tz="UTC")))
-  expect_that(ymd_hms("2012-03-04T05:06:07Z", tz="America/Chicago"), 
+  expect_that(ymd_hms("2012-03-04T05:06:07Z", tz="America/Chicago"),
               equals(ymd_hms("2012-03-03 23:06:07", tz="America/Chicago")))
-  
+
   ## check for message
-  expect_that(ymd_hms("2012-03-04T05:06:07Z", tz="America/Chicago"), 
-              shows_message("Date in ISO8601 format"))  
+  expect_that(ymd_hms("2012-03-04T05:06:07Z", tz="America/Chicago"),
+              shows_message("Date in ISO8601 format"))
 })
 
 test_that("ymd_hms parses OO and Oo formats correctly", {
   ## +00:00
-  expect_that(ymd_hms("2012-03-04T05:06:07+00:00"), 
+  expect_that(ymd_hms("2012-03-04T05:06:07+00:00"),
               equals(ymd_hms("2012-03-04 05:06:07", tz="UTC")))
   ## -HH
-  expect_that(ymd_hms("2012-03-04T05:06:07-01"), 
+  expect_that(ymd_hms("2012-03-04T05:06:07-01"),
               equals(ymd_hms("2012-03-04 06:06:07", tz="UTC")))
   ## -HHMM
-  expect_that(ymd_hms("2012-03-04T05:06:07-0130"), 
+  expect_that(ymd_hms("2012-03-04T05:06:07-0130"),
               equals(ymd_hms("2012-03-04 06:36:07", tz="UTC")))
   ## -HH:MM
-  expect_that(ymd_hms("2012-03-04T05:06:07-01:30"), 
+  expect_that(ymd_hms("2012-03-04T05:06:07-01:30"),
               equals(ymd_hms("2012-03-04 06:36:07", tz="UTC")))
   ## +HH
-  expect_that(ymd_hms("2012-03-04T05:06:07+01"), 
+  expect_that(ymd_hms("2012-03-04T05:06:07+01"),
               equals(ymd_hms("2012-03-04 04:06:07", tz="UTC")))
   ## +HHMM
-  expect_that(ymd_hms("2012-03-04T05:06:07+0130"), 
+  expect_that(ymd_hms("2012-03-04T05:06:07+0130"),
               equals(ymd_hms("2012-03-04 03:36:07", tz="UTC")))
   ## +HH:MM
-  expect_that(ymd_hms("2012-03-04T05:06:07+01:30"), 
+  expect_that(ymd_hms("2012-03-04T05:06:07+01:30"),
               equals(ymd_hms("2012-03-04 03:36:07", tz="UTC")))
   ## vectorizes
-  expect_that(ymd_hms(c("2012-03-04T05:06:07+01", "2012-03-04T05:06:07+01:30")), 
-              equals(ymd_hms(c("2012-03-04 04:06:07", "2012-03-04 03:36:07"), tz="UTC")))  
+  expect_that(ymd_hms(c("2012-03-04T05:06:07+01", "2012-03-04T05:06:07+01:30")),
+              equals(ymd_hms(c("2012-03-04 04:06:07", "2012-03-04 03:36:07"), tz="UTC")))
 
-  expect_that(ymd_hms("2012-03-04T05:06:07-01:30", tz="America/Chicago"), 
+  expect_that(ymd_hms("2012-03-04T05:06:07-01:30", tz="America/Chicago", quiet = T),
               equals(ymd_hms("2012-03-04 00:36:07", tz="America/Chicago")))
 
-  expect_that(ymd_hms("2012-03-04T05:06:07-01:30", tz="America/Chicago"), 
-              shows_message("Date in ISO8601 format"))  
+  expect_that(ymd_hms("2012-03-04T05:06:07-01:30", tz="America/Chicago"),
+              shows_message("Date in ISO8601 format"))
 })
 
 test_that("ymd_hms parses mixed ISO-8601/non-ISO-8601 formats",{
-  expect_that(ymd_hms(c("2012-03-04T05:06:07Z", "2001-02-03 04:05:06"), 
-                      tz="America/Chicago"), 
-              equals(ymd_hms(c("2012-03-03 23:06:07", "2001-02-03 04:05:06"), 
+  expect_that(ymd_hms(c("2012-03-04T05:06:07Z", "2001-02-03 04:05:06"),
+                      tz="America/Chicago"),
+              equals(ymd_hms(c("2012-03-03 23:06:07", "2001-02-03 04:05:06"),
                              tz="America/Chicago")))
 })
 
+test_that("parse_date_time2 and fast_strptime parse ISO8601 timezones",{
+    tm <- "2001-02-03 11:22:33-0630"
+    ptm <- as.POSIXct("2001-02-03 17:52:33", tz = "UTC")
+    expect_equal(fast_strptime(tm, "%Y-%m-%d %H:%M:%S%Oz"), ptm)
+    expect_equal(fast_strptime(tm, "%Y-%m-%d %H:%M:%S%z"), ptm)
+    expect_equal(fast_strptime(tm, "%Y-%m-%d %H:%M:%OS%Oz"), ptm)
+    expect_equal(fast_strptime(tm, "%Y-%m-%d %H:%M:%OS%z"), ptm)
+    expect_equal(parse_date_time2(tm, "YmdHMSz"), ptm)
+    expect_equal(parse_date_time2(tm, "YmdHMSOz"), ptm)
+    expect_equal(parse_date_time2(tm, "YmdHMOSz"), ptm)
+    expect_equal(parse_date_time2(tm, "YmdHMOSOz"), ptm)
 
-## c("2012-12-04 15:06:06.952000-08:00", "2012-12-04 15:04:01.640000-08:00", 
-##   "2012-12-02 17:58:31.141000-08:00", "2012-12-04 17:15:14.091000-08:00", 
+    tm <- "2001-02-03 11:22:33-06:30"
+    expect_equal(fast_strptime(tm, "%Y-%m-%d %H:%M:%OS%OO"), ptm)
+    expect_equal(fast_strptime(tm, "%Y-%m-%d %H:%M:%OS%z"), ptm)
+    expect_equal(parse_date_time2(tm, "YmdHMSz"), ptm)
+    expect_equal(parse_date_time2(tm, "YmdHMSOO"), ptm)
+})
+
+
+test_that("ymd_hms, parse_date_time2 and fast_strptime give the same result", {
+  ## random times between 1400 and 3000
+  X <- as.character(.POSIXct(runif(1000, -17987443200, 32503680000)))
+  oposix <- as.POSIXct(X, tz = "UTC")
+  opdt1 <- ymd_hms(X)
+  opdt2 <- parse_date_time2(X, "YmdHMOS")
+  ostrptime <- fast_strptime(X, "%Y-%m-%d %H:%M:%OS")
+  expect_equal(oposix, opdt1)
+  expect_equal(oposix, opdt2)
+  expect_equal(oposix, ostrptime)
+})
+
+test_that("fast_strptime and parse_date_time2 parse correctly verbose formats", {
+  expect_equal(fast_strptime("aa 2000 bbb10ccc 12 zzz", "aa %Y bbb%mccc %d zzz"),
+               as.POSIXct("2000-10-12", tz = "UTC"))
+  expect_equal(fast_strptime("aa 2000 5555 bbb10ccc 12 zzz", "aa %Y 5555 bbb%mccc %d zzz"),
+               as.POSIXct("2000-10-12", tz = "UTC"))
+
+  expect_equal(parse_date_time2("aa 2000 bbb10ccc 12 zzz", "Ymd"),
+               as.POSIXct("2000-10-12", tz = "UTC"))
+  expect_equal(parse_date_time2("aa 2000 5555 bbb10ccc 12 zzz", "Ymd"),
+               as.POSIXct(as.POSIXlt(NA, tz = "UTC")))
+})
+
+
+## c("2012-12-04 15:06:06.952000-08:00", "2012-12-04 15:04:01.640000-08:00",
+##   "2012-12-02 17:58:31.141000-08:00", "2012-12-04 17:15:14.091000-08:00",
 ##   "2012-12-04 17:16:05.097000-08:00")
 
-
 ## ### speed:
 ## options(digits.secs = 3)
-## tt <- c(rep(as.character(Sys.time()), 1e5))#, "sfds", "sdfdsf")
-
+## ## random times between 1400 and 3000
+## tt <- as.character(.POSIXct(runif(1e6, -17987443200, 32503680000)))
 ## system.time(out <- as.POSIXct(tt, tz = "UTC"))
-## system.time(out <- ymd_hms(tt))
+## system.time(out1 <- ymd_hms(tt)) ## format learning overhead
+## system.time(out2 <- parse_date_time2(tt, "YmdHMOS"))
+## system.time(out3 <- fast_strptime(tt, "%Y-%m-%d %H:%M:%OS"))
+## all.equal(out, out1)
+## all.equal(out, out2)
+## all.equal(out, out3)
+
+## this one is very slow
+## system.time(out <- ymd_hms(tt, tz = "America/Chicago"))
 
 ## ttz <- paste(tt, "-0600", sep = "")
-## system.time(parse_date_time(ttz, "YmdHMOSz"))
-## head(parse_date_time(ttz, "YmdHMOSz"))
+## system.time(out <- parse_date_time(ttz, "YmdHMOSz"))
+## system.time(out <- ymd_hms(ttz))
+## head(ttz)
+## head(out)
+## system.time(out2 <- parse_date_time2(ttz, "YmdHMOSz"))
+## system.time(out2 <- parse_date_time2(ttz, "YmdHMOSOz"))
+## system.time(out3 <- fast_strptime(ttz, "%Y-%m-%d %H:%M:%OS%z"))
+## all.equal(out, out2)
+## all.equal(out, out3)
+
+## fast_strptime(tm, "%Y-%m-%d %H:%M:%S%z")
+
+## fast_strptime("2893-04-08 22:54:37+0635", "%Y-%m-%d %H:%M:%S%Oz")
+
+
 ## system.time(out <- ymd_hms(ttz, tz = ""))
 
 ## stamp_OO <- stamp("2013-01-01T00:00:00-06:00")
@@ -507,90 +564,3 @@ test_that("ymd_hms parses mixed ISO-8601/non-ISO-8601 formats",{
 ## system.time(out <- ymd_hms(tt))
 ## Rprof(NULL)
 ## summaryRprof()
-
-## tt <- rep("3:3:3", 10^5)
-## system.time(out <- hms(tt))
-
-
-## ## ** heterogenuous formats **
-## x <- c(20100101120101, "dsf 09-01-02 12-01-02 dff", "2009.01.03 12:01:03",
-##        "2009-1-4 12-1-4",
-##        "2009-1, 5 12:1, 5",
-##        "200901-08 1201-08",
-##        "2009 arbitrary 1 non-decimal 6 chars 12 in between 1 !!! 6",
-##        "OR collapsed formats: 20090107 120107 (as long as prefixed with zeros)",
-##        "Automatic wday, Thu, detection, 10-01-10 10:01:10 and p format: AM sdf",
-##        "Created on 10-01-11 at 10:01:11 PM")
-## ymd_hms(x)
-
-## guess_formats(x, "ymdHMS")
-
-## tt <- rep.int(x, 1e4)
-## system.time(as.POSIXct(tt))
-## system.time(ymd_hms(tt))
-
-
-
-## ## ** stamp ** functionality:
-## Sys.setlocale("LC_TIME", "en_US.utf8")
-
-## y <- c('February 20th 1973',
-##        "february  14, 2004",
-##        "01 3 2010",
-##        "1 3 10",
-##        "12/31/99", 
-##        "DOB:12/11/00", 
-##        'Thu, 1 July 2004 22:30:00',
-##        "1979-05-27 05:00:59",
-##        '00/13/10',
-##        "03:23:22 pm")
-## D <- as.POSIXct("2012-08-13 11:37:53", tz = "UTC")
-## stamp(y[[7]])(D)
-## cbind(y, unlist(lapply(y, function(x) stamp(x, quiet = TRUE)(D))))
-
-## tt <- c(rep.int(as.character(Sys.Date()), 55),
-##         rep.int(as.character(Sys.time()), 55))
-
-## ## locales
-## Sys.setlocale("LC_TIME", "zh_CN.utf8")
-## format(Sys.time(), format = "%A %Y %B %d %I:%M:%S %p")
-
-## Sys.setlocale("LC_TIME", "ru_RU.utf8")
-## format(Sys.time(), format = "%a %Y %b %d %I:%M:%S %p")
-
-## x_CN <- "星期二 2012 八月 14 12:07:40 下午"
-## ymd_hms(x_CN, locale = "zh_CN.utf8")
-## guess_formats(x_CN, "ymdT", locale = "zh_CN.utf8")
-## x_RO <- "Ma 2012 august 14 11:28:30 "
-## ymd_hms(x_RO, locale = "ro_RO.utf8")
-## x_RU <- "Вт. 2012 Август 14 11:28:19 "
-## x_RU2 <- "Вт. 2012 авг. 14 11:52:57 "
-## ymd_hms(c(x_RU, x_RU2), locale = "ru_RU.utf8")
-
-
-## gsub("([].|(){^$*+?[])", "\\\\\\1", "sdfs.")
-
-## ## ** truncated time-dates **
-## x <- c("2011-12-31 12:59:59", "2010-01-01 12:11", "2010-01-01 12", "2010-01-01")
-## ymd_hms(x, missing = 3)
-
-## ## x <- c("2011-12-31 12:59", "2010-01-01 12", "2010-01-01")
-## ## ymd_hm(x, missing = 2)
-
-
-## ## ** fractional seconds **
-## library(devtools)
-## install_github("lubridate", "vitoshka")
-## library(lubridate)
-
-## options(digits.secs = 3)
-## x <- c("2011-12-31 12:59:59.23", "2010-01-01 12:11:10")
-## ymd_hms(x)
-## hms("3:3:3.34")
-
-## ## ## New formats:
-## ymd_hms("10-12-01 01:01:01 AM", "10-12-01 01:01:01 PM", 101010202020)
-## guess_formats("101010202020", "ymdT")
-## guess_formats("001020", "ymd")
-
-## guess_formats("10 am", "r")
